@@ -56,6 +56,7 @@
 void print_usage(void);
 void print_help(fluid_settings_t *settings);
 void print_welcome(void);
+void print_configure(void);
 
 #if !defined(MACINTOSH)
 static fluid_cmd_handler_t* newclient(void* data, char* addr);
@@ -528,7 +529,7 @@ int main(int argc, char** argv)
       else fluid_settings_setstr(settings, "audio.file.type", optarg);
       break;
     case 'V':
-      printf("FluidSynth %s\n", VERSION);
+      print_configure();
       exit (0);
       break;
     case 'v':
@@ -842,11 +843,23 @@ print_usage()
 void
 print_welcome()
 {
-  printf("FluidSynth version %s\n"
+  printf("FluidSynth runtime version %s\n"
 	 "Copyright (C) 2000-2018 Peter Hanappe and others.\n"
 	 "Distributed under the LGPL license.\n"
 	 "SoundFont(R) is a registered trademark of E-mu Systems, Inc.\n\n",
-	 FLUIDSYNTH_VERSION);
+	 fluid_version_str());
+}
+
+void print_configure()
+{
+    printf("FluidSynth executable version %s\n", FLUIDSYNTH_VERSION);
+    puts("Sample type="
+#ifdef WITH_FLOAT
+    "float"
+#else
+    "double"
+#endif
+    );
 }
 
 /*
