@@ -2008,6 +2008,7 @@ fluid_sample_import_sfont(fluid_sample_t* sample, SFSample* sfsample, fluid_defs
 /*        FLUID_LOG(FLUID_WARN, "Fixing sample %s: at least 8 data points required after loop end", sample->name);     */
 /*        sample->loopend = sample->end - 8; */
 /*      } */
+    sample->valid = TRUE;
   }
   return FLUID_OK;
 }
@@ -3333,11 +3334,39 @@ fixup_sample (SFData * sf)
   int invalid_loops=FALSE;
   int invalid_loopstart;
   int invalid_loopend, loopend_end_mismatch;
+  unsigned int total_bytes = sf->samplesize;
+  unsigned int total_samples = total_bytes / FLUID_MEMBER_SIZE(fluid_defsfont_t, sampledata[0]);
 
   p = sf->sample;
   while (p)
     {
+      unsigned int max_end;
+
       sam = (SFSample *) (p->data);
+
+      /* Standard SoundFont files (SF2) use sample word indices for sample start and end pointers,
+       * but SF3 files with Ogg Vorbis compression use byte indices for start and end. */
+      max_end = (sam->sampletype & FLUID_SAMPLETYPE_OGG_VORBIS) ? total_bytes : total_samples;
+
+      /* ROM samples are unusable for us by definition, so simply ignore them. */
+      if (sam->sampletype & FLUID_SAMPLETYPE_ROM)
+      {
+        sam->start = sam->end = sam->loopstart = sam->loopend = 0;
+        goto next_sample;
+      }
+
+      /* If end is over the sample data chunk or sam start is greater than 4
+       * less than the end (at least 4 samples).
+       *
+       * FIXME: where does this number 4 come from? And do we need a different number for SF3 files?
+       * Maybe we should check for the minimum Ogg Vorbis headers size? */
+      if ((sam->end > max_end) || (sam->start > (sam->end - 4)))
+      {
+        FLUID_LOG (FLUID_WARN, _("Sample '%s' start/end file positions are invalid,"
+                   " disabling and will not be saved"), sam->name);
+        sam->start = sam->end = sam->loopstart = sam->loopend = 0;
+        goto next_sample;
+      }
       
       /* The SoundFont 2.4 spec defines the loopstart index as the first sample point of the loop */
       invalid_loopstart = (sam->loopstart < sam->start) || (sam->loopstart >= sam->loopend);
@@ -3345,25 +3374,12 @@ fixup_sample (SFData * sf)
        * this is as it should be. however we cannot be sure whether any of sam.loopend or sam.end
        * is correct. hours of thinking through this have concluded, that it would be best practice 
        * to mangle with loops as little as necessary by only making sure loopend is within
-       * sdtachunk_size. incorrect soundfont shall preferably fail loudly. */
-      invalid_loopend = (sam->loopend > sdtachunk_size) || (sam->loopstart >= sam->loopend);
+       * max_end. incorrect soundfont shall preferably fail loudly. */
+      invalid_loopend = (sam->loopend > max_end) || (sam->loopstart >= sam->loopend);
       
       loopend_end_mismatch = (sam->loopend > sam->end);
-	  
-      /* if sample is not a ROM sample and end is over the sample data chunk
-         or sam start is greater than 4 less than the end (at least 4 samples) */
-      if ((!(sam->sampletype & FLUID_SAMPLETYPE_ROM) && sam->end > sdtachunk_size)
-          || sam->start > (sam->end - 4))
-      {
-          FLUID_LOG (FLUID_WARN, _("Sample '%s' start/end file positions are invalid,"
-          " disabling and will not be saved"), sam->name);
 
-	        /* disable sample by setting all sample markers to 0 */
-	        sam->start = sam->end = sam->loopstart = sam->loopend = 0;
-
-	        return (OK);
-	    }
-      else if (sam->sampletype & FLUID_SAMPLETYPE_OGG_VORBIS)
+      if (sam->sampletype & FLUID_SAMPLETYPE_OGG_VORBIS)
 	    {
             /*
              * compressed samples get fixed up after decompression
@@ -3394,8 +3410,7 @@ fixup_sample (SFData * sf)
              * valid sample will be played */
             sam->loopend = sam->end;
 	        }
-	  
-          if(loopend_end_mismatch)
+          else if(loopend_end_mismatch)
 	        {
             FLUID_LOG (FLUID_DBG, _("Sample '%s' has invalid loop stop '%d',"
               " sample stop at '%d', using it anyway"), sam->name, sam->loopend, sam->end);
@@ -3407,6 +3422,7 @@ fixup_sample (SFData * sf)
       sam->loopstart -= sam->start;
       sam->loopend -= sam->start;
 
+next_sample:
       p = fluid_list_next (p);
     }
 
@@ -3418,6 +3434,7 @@ fixup_sample (SFData * sf)
   return (OK);
 }
 
+
 /*=================================sfont.c========================
   Smurf SoundFont Editor
   ================================================================*/
